use serde::{Deserialize, Serialize};
// use serde_json::Result;
use anyhow::{Context, Result};
use std::collections::HashMap;
use std::fs;
use std::{fs::File, io::Write};

#[derive(Serialize, Deserialize)]
pub struct Configuration {
    pub user_agent: String,
    pub versions: HashMap<String, String>,
}

pub fn read_config(filename: &str) -> Result<Configuration> {
    let data = fs::read_to_string(filename)
        .with_context(|| format!("Failed reading filename {}", filename))?;
    let config: Configuration = serde_json::from_str(&data)?;
    Ok(config)
}

pub fn write_config(filename: &str, config: &Configuration) -> Result<()> {
    let data = serde_json::to_string_pretty(config)?;
    let mut file = File::create(filename)?;
    file.write_all(data.as_bytes())?;
    Ok(())
}

#[derive(Serialize, Deserialize)]
pub struct BasicJSON {
    pub books: Vec<String>,
    pub book: HashMap<String, BasicChaptersJSON>,
}

impl BasicJSON {
    pub fn new() -> Self {
        Self {
            books: Vec::new(),
            book: HashMap::new(),
        }
    }

    /// Add new book, return mutable instance of it.
    pub fn new_book(&mut self, name: &String) -> &mut BasicChaptersJSON {
        if ! self.book.contains_key(name) {
            self.book.insert(name.clone(), BasicChaptersJSON::new());
        }
        self.book.get_mut(name).unwrap()
    }

    pub fn add_to_books(&mut self, name: &str) {
        self.books.push(name.to_string());
    }
}

#[derive(Serialize, Deserialize)]
pub struct BasicChaptersJSON {
    pub chapters: Vec<BasicVersesJSON>,
}

impl BasicChaptersJSON {
    pub fn new() -> Self {
        Self {
            chapters: Vec::new()
        }
    }

    pub fn new_chapter(&mut self, index: usize) -> &mut BasicVersesJSON {
        while self.chapters.len() < index {
            self.chapters.push(BasicVersesJSON::new());
        }
        self.chapters.get_mut(index-1).unwrap()
    }
}

#[derive(Serialize, Deserialize, Debug)]
pub struct BasicVersesJSON {
    pub verses: Vec<BasicVerseJSON>,
}

impl BasicVersesJSON {
    pub fn new() -> Self {
        Self {
            verses: Vec::new()
        }
    }

    pub fn new_verse(&mut self, index: usize) -> &mut BasicVerseJSON {
        while self.verses.len() < index {
            self.verses.push(BasicVerseJSON::new());
        }
        self.verses.get_mut(index-1).unwrap()
    }
}

#[derive(Serialize, Deserialize, Debug)]
pub struct BasicVerseJSON {
    pub start_paragraph: bool,
    pub heading: Option<String>,
    pub verse: String,
}

impl BasicVerseJSON {
    pub fn new() -> Self {
        Self {
            start_paragraph: false,
            heading: None,
            verse: String::new(),
        }
    }
}

/*
impl BasicJSON {
    pub fn new() -> Self {
        Self { book: HashMap::new()}
    }
}
*/

/*
#[derive(Serialize, Deserialize)]
pub struct BasicVerseJSON {
    pub verse: u8,
    pub text: String,
}
*/

pub fn save_basic_json(filename: &str, json: &BasicJSON) -> Result<()> {
    let data = serde_json::to_string_pretty(json)?;
    let mut file = File::create(filename)?;
    file.write_all(data.as_bytes())?;
    Ok(())
}
